-- ========================================================================== --
-- Useful constants
-- ========================================================================== --
-- Misc
local IMPROVEDMB_STEP = 0.5

-- Historize recorded fight
function ShadowFriend:historyPush()
	-- Push all history records for the encounter one step further
	local historyBoss = self.db.profile.history[self.recordingBossName]
	if historyBoss then
		for i = #historyBoss,1,-1  do
			if i < self.db.profile.historySize then
				historyBoss[i+1] = historyBoss[i]
			end
		end
	else
		self.db.profile.history[self.recordingBossName] = {}
		historyBoss = self.db.profile.history[self.recordingBossName]
	end
	
	historyBoss[1] = {}
	
	local historyEncounter = historyBoss[1]
	
	historyEncounter.bossName = self.recordingBossName
	historyEncounter.length = self.recordingStop - self.recordingStart
	historyEncounter.date = date("%x", self.recordingStart) 
	historyEncounter.time = date("%X", self.recordingStart)
	historyEncounter.actor = {}
	for name, stat in pairs(self.recordingStat) do
		historyEncounter.actor[name] = {}
		local statForActor = historyEncounter.actor[name]
		-- Mana return efficiency
		if stat.shadowDmg ~= 0 then
			statForActor.manaReturnEfficiency = stat.vtManaGained / (stat.shadowDmg * 0.05)
		else
			statForActor.manaReturnEfficiency = 0
		end
		
		-- Mind Blast cooldown management efficiency
		local improvedMBInvestment = 5 -- If it wasn't possible to get talent investment we assume 5/5 Improved Mind Blast
		if self.improvedMBInvestment[name] then
			improvedMBInvestment = self.improvedMBInvestment[name]
		else
			self:debug("Improved Mind Blast investment not found for : " .. name, true)
		end
		-- We assume 1.5sec cast time (haste not taken into account)
		local mbCycleLength = 8 + 1.5 - improvedMBInvestment * IMPROVEDMB_STEP
		statForActor.mbEfficiency = stat.mbNumber / (historyEncounter.length / mbCycleLength)
		
		-- Shadow Word: Death cooldown management efficiency
		local swdCycleLength = 12
		statForActor.swdEfficiency = stat.swdNumber / (historyEncounter.length / swdCycleLength)
		
		-- Hazardous Shadow Word: Death number
		statForActor.swdNumber = stat.swdNumber
		statForActor.hazardousSwdNumber = stat.hazardousSwdNumber
			
		-- Deaths
		statForActor.deaths = stat.deaths
	end
end

function ShadowFriend:deleteHistorySelectedBoss()
	self.db.profile.history[self.db.profile.historySelectedBoss] = nil
	for encounterName, _ in pairs(self.db.profile.history) do
		self.db.profile.historySelectedBoss = encounterName
		break
	end
end

function ShadowFriend:deleteHistoryEncounter(bossName, encounterIndex)
	local encounters = self.db.profile.history[bossName]
	for i = encounterIndex, #encounters - 1 do
		encounters[i] = encounters[i+1] 
	end
	encounters[#encounters] = nil
end
